<?php

/**
 *
 * @package Plugin
 * @subpackage Redirect301
 * @author Michael Loy
 * @copyright LOY GmbH <www.loy.info>
 * @license http://www.contenido.org/license/LIZENZ.txt
 * @link https://www.loy.info
 * @link http://www.contenido.org
 */



function redirect301AfterInitializeVariables(){
    global $cfg,$lang,$idart;

    if(!empty($idart) AND !empty($lang)){
        $start = new contenidoRedirect301($idart,$lang);
    }

}


class contenidoRedirect301 {

    var $serverUrl;


    function __construct($idart,$lang){
        global $cfg;

        $rdb = cRegistry::getDb();

        $this->serverUrl = (isset($_SERVER['HTTPS']) ? "https" : "http") . "://$_SERVER[HTTP_HOST]";


        //initialize if there is no entry
        $query = "SELECT COUNT(idart) AS count FROM ". $cfg['tab']['redirect301']['redirect'] ." LIMIT 0,1";
        $rdb->query($query);
        while ( $rdb->next_record() ) {

            if($rdb->f("count") > 0){

                $this->checkRedirect($idart,$lang);

            }else{
                //no rewrites yet, initialize
                $this->initializeRewrites();
            }
        }

    }

    //generate default db entries
    function initializeRewrites(){
        global $cfg;

        $rdb = cRegistry::getDb();
        $rdb2 = cRegistry::getDb();

        $query = "SELECT idart, idlang FROM ". $cfg['tab']['art_lang'];

        $rdb->query($query);

        while ( $rdb->next_record() ) {
            $query2 = "INSERT INTO ". $cfg['tab']['redirect301']['redirect'] ." SET idart = ". $rdb->f("idart") .", idlang = ". $rdb->f("idlang") .", urlpath = '". $this->getUrl($rdb->f("idart"),$rdb->f("idlang")) ."', is_current = 1 ";
            $rdb2->query($query2);
        }

    }

    //returns absolute URL
    function getUrl($idart,$lang){
        global $cfg;

        $categoryId = $this->getCategoryId($idart);
        $resIsStartArticle = $this->isStartArticle($idart,$lang);

        if( $resIsStartArticle == true AND $this->isFirstRootCat($categoryId) ){
            return "/";
        }

        if( $resIsStartArticle == true AND $this->isRootCat($categoryId) == false  )  {
            $params['idcat'] = $categoryId;
        }else{
            $params['idart'] = $idart;
        }
        $params['lang'] = $lang;
        $params['changelang'] = $lang;

        if(strpos($cfg["version"], '4.9') !== false){
            //4.9

            $newUrl = cUri::getInstance()->build($params, true);

            return $this->urlStrip($newUrl);

        }else{
            //4.8

            $urlBuilder = Contenido_Url::getInstance();
            $newUrl = $urlBuilder->build($params);

            return $this->urlStrip($newUrl);

        }

    }


    //strips complete URL for db insert
    function urlStrip($url){
        $newUrl = str_replace($this->serverUrl,"",$url);
        return $newUrl;
    }


    //301 redirect
    function redirectToCurrent($urlpath){

        header("HTTP/1.1 301 Moved Permanently");
        header("Location: ". $this->serverUrl . $urlpath);
        header("Connection: close");
        exit();

    }


    //check if article is an startarticle
    function isStartArticle($idart,$lang){
        global $cfg;

        $rdb = cRegistry::getDb();

        $query = "SELECT al.idartlang, cl.startidartlang FROM ". $cfg['tab']['art_lang'] ." AS al
                    LEFT JOIN ". $cfg['tab']['cat_lang'] ." AS cl
                    ON cl.startidartlang = al.idartlang
                    WHERE al.idart = ". $idart ." AND al.idlang = ". $lang ." LIMIT 0,1";

        $rdb->query($query);

        while($rdb->next_record()){

            if($rdb->f("idartlang") == $rdb->f("startidartlang")){
                return true;
            }else{
                return false;
            }
        }
        return false;
    }


    //returns idcat of an article
    function getCategoryId($idart){
        global $cfg;

        $rdb = cRegistry::getDb();

        $query = "SELECT idcat FROM ". $cfg['tab']['cat_art'] ."
                    WHERE idart = ". $idart ." ";
        $rdb->query($query);

        while($rdb->next_record()){
            return $rdb->f("idcat");
        }

    }


    function checkRedirect($idart,$lang){
        global $cfg,$cfgClient,$client;

        $rdb = cRegistry::getDb();
        $rdb2 = cRegistry::getDb();
        //Do nothing, if URL is OK


        $sql = "SELECT * FROM ". $cfg['tab']['redirect301']['redirect'] ."
                    WHERE urlpath = '". $_SERVER["REQUEST_URI"] ."'";

        $rdb->query($sql);
        while( $rdb->next_record() ){

            $currentUrlPath = $this->getUrl($rdb->f("idart"),$rdb->f("idlang"));

            if($rdb->f("is_current") == 1){

                if($currentUrlPath != $_SERVER["REQUEST_URI"] ){
                    //old URL is new again -> switch

                    $this->resetRedirects($rdb->f("idart"),$rdb->f("idlang"));

                    $sql2 = "UPDATE ". $cfg['tab']['redirect301']['redirect'] ."
                    SET is_current = 1 WHERE urlpath = '". $currentUrlPath ."' AND idart = ". $rdb->f("idart") ." AND idlang = ". $rdb->f("idlang");

                    $rdb2->query($sql2);

                    $this->redirectToCurrent( $currentUrlPath );

                }else{
                    //everything is fine
                }

            }else{

                if($currentUrlPath == $_SERVER["REQUEST_URI"] ){
                    //old URL is new again -> switch

                    $this->resetRedirects($rdb->f("idart"),$rdb->f("idlang"));

                    $sql2 = "UPDATE ". $cfg['tab']['redirect301']['redirect'] ."
                    SET is_current = 1 WHERE urlpath = '". $_SERVER["REQUEST_URI"] ."' AND idart = ". $rdb->f("idart") ." AND idlang = ". $rdb->f("idlang");

                    $rdb2->query($sql2);

                }else{

                    //redirect to new URL
                    $sql2 = "SELECT urlpath FROM ". $cfg['tab']['redirect301']['redirect'] ."
                    WHERE idart = ". $rdb->f("idart") ." AND is_current = 1";

                    $rdb2->query($sql2);
                    while($rdb2->next_record()){
                        $this->redirectToCurrent($rdb2->f("urlpath"));
                        return true;
                    }
                }
            }
        }




        //Add new URL if there is a new one
        $sql = "SELECT idart, idlang FROM ". $cfg['tab']['redirect301']['redirect'] ."
                    WHERE idart = ". $idart ." AND is_current = 1";


        $rdb->query($sql);
        while($rdb->next_record()){


            if($this->getUrl($rdb->f("idart"),$rdb->f("idlang")) == $_SERVER["REQUEST_URI"] ){
                //everything the same, do nothing
                return true;

            }else{

                //404
                header("HTTP/1.0 404 Not Found");
                $ch = curl_init();
                curl_setopt($ch, CURLOPT_URL, $this->serverUrl . $this->getUrl($cfgClient[$client]["errsite"]["idart"],$lang));
                curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
                curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
                $result = curl_exec($ch);
                curl_close($ch);
                echo $result;
                exit();


            }

        }

        //new page
        $newUrl = $this->getUrl($idart,$lang);
        $this->addRedirect($idart,$lang,$newUrl);

        return false;

    }


    function addRedirect($idart,$lang,$newUrl){
        global $cfg;

        $rdb = cRegistry::getDb();
        $rdb2 = cRegistry::getDb();

        $this->resetRedirects($idart,$lang);

        $query = "SELECT idrewrite FROM ". $cfg['tab']['redirect301']['redirect'] ." WHERE
                    idart = ". $idart .",
                    idlang = ". $lang .",
                    urlpath = '". $newUrl ."'
                    ";

        $rdb->query($query);

        $rdb->next_record();

        if( empty($rdb->f("idrewrite")) ){

            $query2 = "INSERT INTO ". $cfg['tab']['redirect301']['redirect'] ." SET
                        idart = ". $idart .",
                        idlang = ". $lang .",
                        urlpath = '". $newUrl ."',
                        is_current = 1
                        ";

            $rdb2->query($query2);

        }else{
            $query2 = "UPDATE ". $cfg['tab']['redirect301']['redirect'] ." SET
                        is_current = 1
                        WHERE idrewrite = ". $rdb->f("idrewrite") ."
                        ";

            $rdb2->query($query2);

        }

        return true;

    }


    function resetRedirects($idart,$lang){
        global $cfg;

        $rdb = cRegistry::getDb();

        $query = "UPDATE ". $cfg['tab']['redirect301']['redirect'] ." SET is_current = 0
                    WHERE idart = ". $idart ." AND idlang = ". $lang;

        $rdb->query($query);

        return true;
    }


    function cleanRedirects(){
        global $cfg;

        $rdb = cRegistry::getDb();

        $query = "DELETE FROM ". $cfg['tab']['redirect301']['redirect'] ." WHERE is_current = 0 ";

        $rdb->query($query);

        return true;
    }


    function isRootCat($idcat){
        global $cfg;

        $rdb = cRegistry::getDb();

        $query = "SELECT idcat FROM ". $cfg['tab']['cat'] ." WHERE idcat = ". $idcat ." AND parentid = 0";

        $rdb->query($query);
        $rdb->next_record();

        if($rdb->f("idcat") > 0){
            return true;
        }else{
            return false;
        }

    }


    function isFirstRootCat($idcat){
        global $cfg;

        $rdb = cRegistry::getDb();

        $query = "SELECT idcat FROM ". $cfg['tab']['cat'] ." WHERE idcat = ". $idcat ." AND preid = 0 AND parentid = 0";

        $rdb->query($query);
        $rdb->next_record();

        if($rdb->f("idcat") > 0){
            return true;
        }else{
            return false;
        }

    }


}


?>
